class TrainerLoopConfig():
    def __init__(self):
        self.train = None

    def process_batch(self, x_batch, y_batch):
        return x_batch, y_batch

    def forward_call(self, model, x_batch):
        return model(*x_batch)

    def prepare_for_optimizer_step(self, model):
        pass

    def prepare_for_metrics_update(self, y_batch_pred, y_batch):
        return y_batch_pred, y_batch


_default_trainer_loop_config = TrainerLoopConfig()


from multiprocessing import cpu_count as _cpu_count

from ._to_tensor import to_tensor
from .data import *


def _get_num_workers(num_workers):
    if num_workers == -1:
        return _cpu_count() - 1
    return num_workers


import torch
from collections import OrderedDict

_optimizers_dict = OrderedDict({
    'adam': (torch.optim.Adam, {'lr': 1e-3}),
    'rmsprop': (torch.optim.RMSprop, {'lr': 1e-3}),
    'sgd': (torch.optim.SGD, {'lr': 1e-2, 'momentum': 0.9})
})

def _create_optimizer(i, model_parameters):
    if isinstance(i, str):
        name = i.lower()
        if name not in _optimizers_dict:
            raise KeyError(f'Invalid name, we support {list(_optimizers_dict.keys())}.')
        optim_class, kwargs = _optimizers_dict[name]
        return optim_class(model_parameters, **kwargs)
    return i

__all__ = []